--[[
************************************************************************
Project				: BetterGuildControl
Author				: zhinjio
Project Revision	: r9
Project Date		: 20090502081204

File				: BetterGuildControl.lua
Commit Author		: zhinjio
Commit Revision		: 9
Commit Date			: 20090502081204
************************************************************************
Description	:
	Main addon file. 

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "BetterGuildControl"
local FULLNAME = "BetterGuildControl"
local BetterGuildControl = LibStub( "AceAddon-3.0" ):NewAddon( MODNAME, "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )
local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")

_G[ MODNAME ] = BetterGuildControl
local addon = BetterGuildControl

local addonversion = "r9"
local addonauthor = "zhinjio"
local builddate = "20090502081204"

local modularOptions = {}
local tooltip
local LTT = LibStub:GetLibrary( "LibQTip-1.0" )

-- addon globals
-- addon.someValue = 0
addon.me = ""
addon.GRUs = 0
addon.dataReady = false
addon.permissionStrings = {
	L["Listen to Guild Chat"],
	L["Speak in Guild Chat"],
	L["Listen to Officer Chat"],
	L["Speak in Officer Chat"],
	L["Promote Member"],
	L["Demote Member"],
	L["Invite Member"],
	L["Remove Member"],
	L["Set MOTD"],
	L["Edit Public Note"],
	L["View Officer Note"],
	L["Edit Officer Note"],
	L["Modify Guild Info"],
	L["Guild Bank Withdraw Repair"],
	L["Guild Bank Withdraw Gold"],
	L["Create Guild Event"]
}

local db

function addon.Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\" .. FULLNAME .. "\\icon:" .. fh .. "\124t"
	if ( msg ~= nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " " .. MODNAME .. " : " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function addon.Debug( msg )
	if ( msg ~= nil ) then
		if ( db.options.display.debug ) then
			addon.Print( "DEBUG: " .. msg )
		end
	end
end

-- coloring agents
local function Orange ( text ) return "|cffdea51c" .. text .. "|r" end
local function Yellow ( text ) return "|cffffff00" .. text .. "|r" end
local function Cyan ( text ) return "|cff00ffff" .. text .. "|r" end

local function HideTooltip()
	tooltip:SetScript( "OnLeave", nil )
	tooltip:Hide()
end

local function drawTooltip()
	tooltip:Hide()
	tooltip:Clear()
	tooltip:AddHeader( Yellow( MODNAME ) )
	tooltip:AddLine( " " )
	local t = {}
	if ( addon.dataReady == true ) then
		tooltip:SetColumnLayout( ( addon.numGuildRanks + 1 ) ) 
		t[1] = L["Rank"]
		for i = 1, addon.numGuildRanks do
			t[i+1] = " " .. i .. " "
		end
		tooltip:AddLine( unpack( t ) )
		tooltip:SetCell( 3, 1, Orange( L["Rank"] ) , "RIGHT" )
		tooltip:AddLine( " " )
		tooltip:SetCell( 4, 1, Orange( L["Permission"] ), "LEFT" )
		for i = 1, 16 do
			t = nil
			t = {}
			t[1] = addon.permissionStrings[i]
			for j = 1, addon.numGuildRanks do
				if ( addon.GuildRanks[j][i] ~= nil ) then
					t[j+1] = Blue( " Y " )
				else
					t[j+1] = Blue( " N " )
				end
			end
			tooltip:AddLine( unpack( t ) )
		end
	else
		tooltip:AddLine( L["Data not yet cached"] )
	end
	-- tooltip display code here...

	tooltip:AddLine(" ")
	local lineNum = tooltip:AddLine( "b" )
	local numCols = tooltip:GetColumnCount()
	tooltip:SetCell( lineNum, 1, L["Left-Click to change displays"], "LEFT", numCols )
	lineNum = tooltip:AddLine( "c" )
	tooltip:SetCell( lineNum, 1, L["Right-click for Configuration"], "LEFT", numCols )
	
	-- Color the even columns - thanks to Toadkiller/Broker_Currency for the code
	local column
	local summaryColorLight = { r = 1, g = 1, b = 1, a = .3 }
	if (summaryColorLight.a > 0) then
		for index = 2, numCols, 2 do
			column = tooltip:AcquireColumn(index)
			column:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = false, tileSize = 16, insets = {left = 0, right = 0, top = 2, bottom = 2},})
			column:SetBackdropColor(summaryColorLight.r, summaryColorLight.g, summaryColorLight.b, summaryColorLight.a)
		end
	end

end

-- LDB stuff
local LDB = LibStub( "LibDataBroker-1.1" )
local launcher = LDB:NewDataObject( MODNAME, {
	type = "launcher",
	text = " ",
	label = FULLNAME,
	icon = "Interface\\Addons\\" .. FULLNAME .. "\\icon",
	OnClick = function( clickedframe, button )
				if button == "RightButton" then
					addon:ShowConfig()
				else
					drawTooltip()
					tooltip:SmartAnchorTo( clickedframe )
					tooltip:Show()
				end
	end,
	OnEnter = function ( self )
		drawTooltip()
		tooltip:EnableMouse()
		tooltip:SmartAnchorTo( self )
		tooltip:Show()
	end,
	OnLeave = function ( self )
		if MouseIsOver( tooltip ) then
			tooltip:SetScript( 'OnLeave', HideTooltip )
		else
			HideTooltip()
		end
	end,
} )

function addon.upgradeDB()
	local olddb = addon.db.factionrealm[addon.me]
	--[[
	if ( olddb.options.display.dbVersion == "1.0" ) then
		-- original version
		olddb.options.display.debug = false
		olddb.options.display.dbVersion = "1.1"
		addon.Print( string.format( L["Database upgraded to %s"], "1.1" ) )
	end
	]]--
end

-- options stuff
local displayOptions = nil
local function giveDisplay()
	if (not displayOptions) then
		displayOptions = {
			order = 2,
			type = "group",
			name = L["Display Options"],
			desc = L["DISPLAY_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 21,
					type	= "header",
					name	= L["Display Options"],
				},
				desc = {
					order	= 22,
					type	= "description",
					name	= L["DISPLAY_OPTIONS_DESC"],
				},
				sound = {
					order	= 23,
					type	= "toggle",
					name	= L["Enable DEBUG"],
					desc	= L["DISPLAY_DEBUG_DESC"],
					get		= function() return db.options.display.debug end,
					set		= function() db.options.display.debug = not db.options.display.debug end,
				},
				tooltipscale = {
					order	= 24,
					type	= "range",
					name	= L["Tooltip Scale"],
					desc	= L["TOOLTIP_SCALE_DESC"],
					min		= .7,
					max		= 1.3,
					step	= .05,
					bigStep = .05,
					get		= function() return db.options.display.tipscale end,
					set		= function( info, v ) 
								db.options.display.tipscale = v
								tooltip:SetScale( v )
							  end,
				},
			},
		}
	end
	return displayOptions
end

local options = nil
local function fullOptions()
	if (not options) then
		options = {
			type = "group",
			name = MODNAME,
			args = {
				general = {
					order	= 1,
					type	= "group",
					name	= L["General Information"],
					desc	= L["GENERAL_INFO_DESC"],
					args	= {
						header1 = {
							order	= 11,
							type	= "header",
							name	= L["General Information"],
						},
						desc = {
							order	= 12,
							type	= "description",
							name	= L["GENERAL_INFO_DESC"],
						},
						author = {
							order	= 13,
							type	= "description",
							name	= L["Author : "] .. addonauthor .. "\n",
						},
						version = {
							order	= 14,
							type	= "description",
							name	= L["Version : "] .. addonversion .. "\n",
						},
						builddate = {
							order	= 15,
							type	= "description",
							name	= L["Build Date : "] .. builddate .. "\n",
						},
						dbversion = {
							order	= 15,
							type	= "description",
							name	= L["Database Version : "] .. db.options.display.dbVersion .. "\n",
						},
						translators = {
							order	= 16,
							type	= "description",
							name	= string.format( L["Helpful Translators (thank you) : %s"], " \n" )
						},
					},
				},
			},
		}
	end
	
	for k,v in pairs(modularOptions) do
		options.args[k] = (type(v) == "function") and v() or v
	end

	return options
end

function addon:SetupOptions()
	AceConfigReg:RegisterOptionsTable(MODNAME, fullOptions)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")

	-- Fill up our modular options...
	self:RegisterModuleOptions("Display", giveDisplay(), L["Display Options"])
end

function addon:RegisterModuleOptions(name, optionsTable, displayName)
	modularOptions[name] = optionsTable
	self.optionsFrame[name] = AceConfigDialog:AddToBlizOptions(MODNAME, displayName, MODNAME, name)
end

function addon:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory( self.optionsFrame["Display"] )
end

function addon:OnInitialize()
	-- Do Frame Creation stuff here
	-- addon:CreateFrames()
end

function addon:GUILD_ROSTER_UPDATE()
	-- It doesn't appear to be necessary to wait for anything but the first GRU
	-- anymore, but just for good measure, I'm going to wait for the second one.
	if ( addon.GRUs < 2 ) then
		addon.GRUs = addon.GRUs + 1
	else
		-- Gather guild info, start with the ranks
		addon.numGuildRanks = GuildControlGetNumRanks()
		addon.GuildRanks = {}
		for i = 1, addon.numGuildRanks do
			-- for each rank, get all the permissions info
			addon.GuildRanks[ i ] = {}
			GuildControlSetRank( i )
			addon.GuildRanks[ i ].Name = GuildControlGetRankName( i )
			local guildchat_listen, guildchat_speak, officerchat_listen, officerchat_speak, promote, demote,
				invite_member, remove_member, set_motd, edit_public_note, view_officer_note, edit_officer_note,
				modify_guild_info, _, withdraw_repair, withdraw_gold, create_guild_event = GuildControlGetRankFlags();
			addon.GuildRanks[ i ][1] = guildchat_listen
			addon.GuildRanks[ i ][2] = guildchat_speak
			addon.GuildRanks[ i ][3] = officerchat_listen
			addon.GuildRanks[ i ][4] = officerchat_speak
			addon.GuildRanks[ i ][5] = promote
			addon.GuildRanks[ i ][6] = demote
			addon.GuildRanks[ i ][7] = invite_member
			addon.GuildRanks[ i ][8] = remove_member
			addon.GuildRanks[ i ][9] = set_motd
			addon.GuildRanks[ i ][10] = edit_public_note
			addon.GuildRanks[ i ][11] = view_officer_note
			addon.GuildRanks[ i ][12] = edit_officer_note
			addon.GuildRanks[ i ][13] = modify_guild_info
			addon.GuildRanks[ i ][14] = withdraw_repair
			addon.GuildRanks[ i ][15] = withdraw_gold
			addon.GuildRanks[ i ][16] = create_guild_event
		end
		addon.dataReady = true
	end
end

function addon:OnEnable()
    -- Called when the addon is loaded
	addon.me = UnitName( "player" )

	self.db = LibStub( "AceDB-3.0" ):New( MODNAME .. "_DB", nil, "Default" )
	-- do we have any data yet; for this toon? If not, init its table
	if not ( self.db.factionrealm[addon.me] ) then
		self.db.factionrealm[addon.me] = {
		-- Your saved variables stuff here
			options = {
				display = {
					debug = false,
					tipscale = 1,
					dbVersion = "1.0",
				},
			},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end
	db = self.db.factionrealm[addon.me]

	-- tooltip setup and scaling
	tooltip = LTT:Acquire( MODNAME.."TT", 3, "LEFT", "CENTER", "RIGHT" ) -- or GameTooltip
	tooltip:SetScale( db.options.display.tipscale )

	addon:SetupOptions()
	-- Event listeners
    addon:RegisterEvent("GUILD_ROSTER_UPDATE")
end

--[[
************************************************************************
CHANGELOG:

5/1/09 : 
	Initial version
************************************************************************
]]--